<!DOCTYPE html>
<html lang="en">
<head>
    <title>Otto</title>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    {{ font_link | safe }}
    <style>{{ css | safe }}</style>
</head>
<body{% block body_attrs %}{% endblock %}>
{% block body %}{% endblock %}
</body>
</html>
